#include "Thread.h"

#include <pthread.h>
#include <iostream>

using namespace std;

/*!
 * Body of the thread.
 * ThreadMain executes run method of the Thread object.
 * @param _params thread parameters
 */
void * threadMain(void * _params) {
	int rc;
    void *ret;
	ThreadParams * params = (ThreadParams *) _params;
	
	rc = params->thread->run();
    ret = (void *)&rc;

    pthread_exit(ret);
    return ret;
}

int Thread::lastid = 0;

Thread::Thread() {
	
	id	   = ++lastid;	
	signal = THREADSIG_STOP;
	status = 0;

	params.thread = this;
	
    pthread_attr_init(&attr);
    pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_JOINABLE);
}

Thread::~Thread() {
}

int Thread::start() {
    
	signal = THREADSIG_RUN;
	
	cout << "Creating thread [" 
         << id 
         << "] ..." 
         << endl;
	
	int rc = pthread_create(&thread, &attr, threadMain, (void *) &params);
	
	if (rc) {
        
		cerr << "ERROR: Failed to create thread [" 
             << id 
             << "], pthread_create() returned " 
             << rc 
             << endl;	
		
	} else {
		
		cout << "Thread [" 
             << id 
             << "] created" 
             << endl;
	}
    
	pthread_attr_destroy(&attr);
	return rc;
}

int Thread::stop() {
    
	signal = THREADSIG_STOP;
	status = 0;
    
    cout << "Joining thread ["
         << id
         << "] ..."
         << endl;
    
    int rc = pthread_join(thread, (void **) &status);
	
	if (rc) {
        
		cerr << "Failed to join thread [" 
             << id 
             << "], pthread_join() returned " 
             << rc 
             << endl;
		
	} else {
        
		cout << "Thread [" 
             << id 
             << "] joined with status " 
             << status 
             << endl;
	}
	
	return rc;
}